#include <gtk/gtk.h>
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "images/gsame.xpm"
#include "images/null.xpm"
#include "images/a1.xpm"
#include "images/b1.xpm"
#include "images/c1.xpm"
#include "images/d1.xpm"
#include "images/e1.xpm"
#include "images/a1_mark.xpm"
#include "images/b1_mark.xpm"
#include "images/c1_mark.xpm"
#include "images/d1_mark.xpm"
#include "images/e1_mark.xpm"
#include "images/a2.xpm"
#include "images/b2.xpm"
#include "images/c2.xpm"
#include "images/d2.xpm"
#include "images/e2.xpm"
#include "images/a2_mark.xpm"
#include "images/b2_mark.xpm"
#include "images/c2_mark.xpm"
#include "images/d2_mark.xpm"
#include "images/e2_mark.xpm"

#define NORMAL_WIDTH 20
#define NORMAL_HEIGHT 10
#define LARGE_WIDTH 25
#define LARGE_HEIGHT 15

GdkPixmap *tiles[21];
GtkWidget *drawing_board;
GtkWidget *status_bar;
gint context_id;
int width = 20, height = 10;
char board[LARGE_WIDTH][LARGE_HEIGHT];
char undo[LARGE_WIDTH][LARGE_HEIGHT];
char orig[LARGE_WIDTH][LARGE_HEIGHT];
int score;
int score_undo;
int mark;
int mark_undo;
char tiles_style = 0;
gchar *scores_table[10][2];
gchar names[10][9];
gchar scores[10][6];
int game_over;


void read_scores()
{
  FILE *scores_file;
  int i;

  for (i = 0; i < 10; i ++) {
    scores_table[i][0] = names[i];
    scores_table[i][1] = scores[i];
    strcpy (names[i], "nobody");
    strcpy (scores[i], "0");
  }
  if (!(scores_file = fopen (SCORES_FILE, "ro"))) {
    g_print ("Unable to open scores file\n");
    return;
  }
  for (i = 0; i < 10; i++) {
    fscanf (scores_file, "%s %s", names[i], scores[i]);
    if (feof(scores_file)) break;
  }
  fclose (scores_file);
}

void write_scores()
{
  FILE *scores_file;
  int i, j;

  for (i = 0; i < 10; i++) {
    if (atoi (scores[i]) < score) {
      for (j = 9; j > i; j--) {
	strcpy (scores[j], scores[j - 1]);
	strcpy (names[j], names[j - 1]);
      }
      snprintf (scores[i], 9, "%i", score);
      strncpy (names[i], getenv ("USER"), 8);
      break;
    }
  }
  if (!(scores_file = fopen (SCORES_FILE, "w"))) {
    g_print ("Unable to open scores file\n");
    return;
  }
  for (i = 0; i < 10; i++) {
    fprintf (scores_file, "%s %s\n", scores_table[i][0], scores_table[i][1]);
  }
  fclose (scores_file);
}

int calc_score (int x)
{
  if (x < 2) return 0;
  return (x * x - 3 * x + 4);
}

void check_game_over ()
{
  int i, j, k, bonus = 1;
  game_over = 1;
  for (i = 0; i < width; i++)
    for (j = 0; j < height; j++) {
      if ((k = board[i][j]) == 0) continue;
      bonus = 0;
      if ((i > 0 && board[i - 1][j] == k) ||
          (i < width - 1 && board[i + 1][j] == k) ||
          (j > 0 && board[i][j - 1] == k) ||
          (j < height - 1 && board[i][j + 1] == k)) game_over = 0;
    }
  if (bonus == 1) score += 3000;
}

void draw_board ()
{
  int x, y, i;
  for (x = 0; x < width; x++) {
    for (y = 0; y < height; y++) {
      if ((i = board[x][y]) != 0 && tiles_style == 1) i += 10;
      gdk_draw_pixmap(drawing_board->window, drawing_board->style->fg_gc[GTK_WIDGET_STATE(drawing_board)], tiles[i], 0, 0, x * 32, y * 32, 32, 32);
    }
  }
}

void update_status ()
{
  gchar text[64];
  if (game_over)
    snprintf (text, 63, "Game Over  -  Score : %i", score);
  else {
    snprintf (text, 63, "Marked : %i (%i Points)  -  Score : %i", mark, calc_score(mark), score);
  }
  gtk_statusbar_pop (GTK_STATUSBAR (status_bar), context_id);
  gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, text);
}

void make_new ()
{
  int x, y;
  time_t t;

  time(&t);
  srand((unsigned int)t);

  for (x = 0; x < width; x++) {
    for (y = 0; y < height; y++) {
      board[x][y] = (char) ((float) 5*rand()/(RAND_MAX+1.0)) + 1;
      undo[x][y] = board[x][y];
      orig[x][y] = board[x][y];
    }
  }
  score = 0;
  score_undo = 0;
  mark = 0;
  mark_undo = 0;
}

void make_undo ()
{
  int i, j;
  for (i = 0; i < width; i++)
    for (j = 0; j < height; j++)
      undo[i][j] = board[i][j];
  score_undo = score;
  mark_undo = mark;
}

int board_select(int x, int y, char c)
{
  int r = 0;
  if (board[x][y] == c) {
    board[x][y] = c + 5;
    r++;
    if (x > 0) r = r + board_select (x - 1, y, c);
    if (x < width - 1) r = r + board_select (x + 1, y, c);
    if (y > 0) r = r + board_select (x, y - 1, c); 
    if (y < height - 1) r = r + board_select (x, y + 1, c);
  }
  return r;
}

/*
int board_select(int x, int y, char c)
{
  int i, j, k, r = 1;
  board[x][y] = c + 5;
  while(1)
  {
    k = 0;
    for (i = 0; i < width; i++)
      for (j = 0; j < height; j++)
        if (board[i][j] == c + 5) {
          if (i > 0 && board[i-1][j] == c) {
            board[i-1][j] = c + 5;
            k++;
          }
          if (i < width - 1 && board[i+1][j] == c) {
            board[i+1][j] = c + 5;
            k++;
          }
          if (j > 0 && board[i][j-1] == c) {
            board[i][j-1] = c + 5;
            k++;
          }
          if (j < height - 1 && board[i][j+1] == c) {
            board[i][j+1] = c + 5;
            k++;
          }
        }
    if (k == 0) break;
    r += k;
  }
  return r;
}
*/

void board_shift_down (int x, int y)
{
  int i;
  for (i = y; i > 0; i--) {
    board[x][i] = board[x][i - 1];
  }
  board[x][0] = 0;
}

void board_shift_left (int x)
{
  int i, j;
  for (i = x; i < width; i++) {
    for (j = 0; j < height; j++) {
      if (i == width - 1) board[i][j] = 0;
      else board[i][j] = board[i + 1][j];
    }
  }
}

static void new_game (GtkWidget *widget, gpointer data)
{
  make_new ();
  game_over = 0;
  draw_board ();
  update_status ();
}

static void restart_game (GtkWidget *widget, gpointer data)
{
  int i, j;
  for (i = 0; i < width; i++)
    for (j = 0; j < height; j++)
      board[i][j] = orig[i][j];
  score = 0;
  mark = 0;
  game_over = 0;
  draw_board ();
  update_status ();
}

static void undo_game (GtkWidget *widget, gpointer data)
{
  int i, j;
  if (game_over) return;
  for (i = 0; i < width; i++)
    for (j = 0; j < height; j++)
      board[i][j] = undo[i][j];
  score = score_undo;
  mark= mark_undo;
  draw_board ();
  update_status ();
}

void modal_close(GtkObject *widget)
{
  gtk_grab_remove(GTK_WIDGET(widget));
  gtk_widget_destroy(GTK_WIDGET(widget));
}

void scores_game(void)
{
  GtkWidget *window;
  GtkWidget *boxv1;
  GtkWidget *button_close;
  GtkWidget *clist;
  GtkWidget *separator;
  int i;
  gchar *titles[2] = {"Name    ", "Score   "};

  read_scores ();
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_grab_add(window);
  gtk_window_set_position(GTK_WINDOW(window),GTK_WIN_POS_CENTER);
  gtk_window_set_title(GTK_WINDOW(window),"High Scores");
  gtk_container_border_width(GTK_CONTAINER(window), 10);

  gtk_signal_connect_object(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(modal_close), GTK_OBJECT(window));
  gtk_signal_connect_object(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(modal_close), GTK_OBJECT(window));

  boxv1 = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(window), boxv1);

  clist = gtk_clist_new_with_titles (2, titles);
  gtk_clist_column_titles_passive (GTK_CLIST (clist));
  for (i = 0; i < 10; i++) {
    gtk_clist_append (GTK_CLIST (clist), scores_table[i]);
  }
  gtk_box_pack_start(GTK_BOX(boxv1), clist, FALSE, FALSE, 0);  
  separator = gtk_hseparator_new();
  gtk_box_pack_start(GTK_BOX(boxv1), separator, FALSE, FALSE, 10);  

  button_close = gtk_button_new_with_label("Close");
  gtk_box_pack_start(GTK_BOX(boxv1), button_close, FALSE, FALSE, 0);  
  GTK_WIDGET_SET_FLAGS(button_close, GTK_CAN_DEFAULT);
  gtk_widget_grab_default(button_close);
  gtk_signal_connect_object(GTK_OBJECT(button_close), "clicked", GTK_SIGNAL_FUNC(modal_close), GTK_OBJECT(window));

  gtk_widget_show_all(window);
}

void info_game(void)
{
  GtkWidget *window;
  GtkWidget *boxv1;
  GtkWidget *button_close;
  GtkWidget *clist;
  GtkWidget *separator;
  gchar *titles[3] = {"Letter   ", "Color   ", "Quantity"};
  gchar *entries[5][3] = {{"A", "Blue", "  "},
                          {"B", "Red", "  "},
                          {"C", "Purple", "  "},
                          {"D", "Yellow", "  "},
                          {"E", "Cyan", "  "}};
  gchar n[5];
  int c[5] = {0,0,0,0,0}, i, j;

  for (i = 0; i < width; i++)
    for (j = 0; j < height; j++)
      if (board[i][j] != 0) c[(board[i][j] - 1) % 5 ]++;

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_grab_add(window);
  gtk_window_set_position(GTK_WINDOW(window),GTK_WIN_POS_CENTER);
  gtk_window_set_title(GTK_WINDOW(window),"Information");
  gtk_container_border_width(GTK_CONTAINER(window), 10);

  gtk_signal_connect_object(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(modal_close), GTK_OBJECT(window));
  gtk_signal_connect_object(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(modal_close), GTK_OBJECT(window));

  boxv1 = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(window), boxv1);

  clist = gtk_clist_new_with_titles (3, titles);
  gtk_clist_column_titles_passive (GTK_CLIST (clist));
  for (i = 0; i < 5; i++) {
    snprintf (n, 4, "%i", c[i]);
    entries[i][2] = n;
    gtk_clist_append (GTK_CLIST (clist), entries[i]);
  }
  gtk_box_pack_start(GTK_BOX(boxv1), clist, FALSE, FALSE, 0);  
  separator = gtk_hseparator_new();
  gtk_box_pack_start(GTK_BOX(boxv1), separator, FALSE, FALSE, 10);  

  button_close = gtk_button_new_with_label("Close");
  gtk_box_pack_start(GTK_BOX(boxv1), button_close, FALSE, FALSE, 0);  
  GTK_WIDGET_SET_FLAGS(button_close, GTK_CAN_DEFAULT);
  gtk_widget_grab_default(button_close);
  gtk_signal_connect_object(GTK_OBJECT(button_close), "clicked", GTK_SIGNAL_FUNC(modal_close), GTK_OBJECT(window));

  gtk_widget_show_all(window);
}

static void exit_game (GtkWidget *widget, gpointer data)
{
  gtk_main_quit ();
}

static void size_normal (GtkWidget *widget, gpointer data)
{
  if (width != NORMAL_WIDTH || height != NORMAL_HEIGHT) {
    width = NORMAL_WIDTH;
    height = NORMAL_HEIGHT;
    make_new ();
    gtk_widget_hide (drawing_board);
    gtk_drawing_area_size (GTK_DRAWING_AREA (drawing_board), width*32, height*32);
    gtk_widget_show (drawing_board);
  }
}

static void size_large (GtkWidget *widget, gpointer data)
{
  if (width != LARGE_WIDTH || height != LARGE_HEIGHT) {
    width = LARGE_WIDTH;
    height = LARGE_HEIGHT;
    make_new ();
    gtk_widget_hide (drawing_board);
    gtk_drawing_area_size (GTK_DRAWING_AREA (drawing_board), width*32, height*32);
    gtk_widget_show (drawing_board);
  }
}

static void style_letters (GtkWidget *widget, gpointer data)
{
  tiles_style = 0;
  draw_board ();
}

static void style_balls (GtkWidget *widget, gpointer data)
{
  tiles_style = 1;
  draw_board ();
}

void about_game(void)
{
  GtkWidget *window;
  GtkWidget *boxv1;
  GtkWidget *button_close;
  GtkWidget *text;
  GtkWidget *separator;
         
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_grab_add(window);
  gtk_window_set_position(GTK_WINDOW(window),GTK_WIN_POS_CENTER);
  gtk_window_set_title(GTK_WINDOW(window),"About Same Game");
  gtk_container_border_width(GTK_CONTAINER(window), 10);

  gtk_signal_connect_object(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(modal_close), GTK_OBJECT(window));
  gtk_signal_connect_object(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(modal_close), GTK_OBJECT(window));

  boxv1 = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(window), boxv1);

  text = gtk_label_new("Same Game");
  gtk_box_pack_start(GTK_BOX(boxv1), text, FALSE, FALSE, 0);  
  separator = gtk_hseparator_new();
  gtk_box_pack_start(GTK_BOX(boxv1), separator, FALSE, FALSE, 10);
  text = gtk_label_new("Version 1.0pre1");
  gtk_box_pack_start(GTK_BOX(boxv1), text, FALSE, FALSE, 0);  
  text = gtk_label_new("Created 1999-10-01");
  gtk_box_pack_start(GTK_BOX(boxv1), text, FALSE, FALSE, 0);
  text = gtk_label_new("Last modified 2012-10-04");
  gtk_box_pack_start(GTK_BOX(boxv1), text, FALSE, FALSE, 0);
  text = gtk_label_new("Writen by Sergey Kiselev");
  gtk_box_pack_start(GTK_BOX(boxv1), text, FALSE, FALSE, 0);
  text = gtk_label_new("sergey@malinov.com");
  gtk_box_pack_start(GTK_BOX(boxv1), text, FALSE, FALSE, 0);  
  separator = gtk_hseparator_new();
  gtk_box_pack_start(GTK_BOX(boxv1), separator, FALSE, FALSE, 10);  

  button_close = gtk_button_new_with_label("Close");
  gtk_box_pack_start(GTK_BOX(boxv1), button_close, FALSE, FALSE, 0);  
  GTK_WIDGET_SET_FLAGS(button_close, GTK_CAN_DEFAULT);
  gtk_widget_grab_default(button_close);
  gtk_signal_connect_object(GTK_OBJECT(button_close), "clicked", GTK_SIGNAL_FUNC(modal_close), GTK_OBJECT(window));

  gtk_widget_show_all(window); 
}

void help_game (void)
{
  GtkWidget *window;
  GtkWidget *vbox, *hbox, *button_box;
  GtkWidget *scrolled;
  GtkWidget *help;
  GtkWidget *separator;
  GtkWidget *button_close;
  GtkTextBuffer *buffer;
  char      help_text[] =\
"Same Game Help\n"\
"==============\n"\
"\n"\
"Gameplay\n"\
"--------\n"\
"\n"\
"  The Same Game is played on a rectangular field which is\n"\
"initially filled with five kinds of tiles placed at random.\n"\
"The goal of the game is to remove as many tiles as possible\n"\
"from the playing field by selecting groups of adjoining tiles\n"\
"of the same color. Tiles that are no longer supported will\n"\
"fall down, and empty columns will be trimmed away by\n"\
"sliding remaing tiles on the right side to the left.\n"\
"\n"\
"\n"\
"Scoring\n"\
"-------\n"\
"\n"\
"  The game gives N^2 - 3*N + 4 points for removing N tiles at once.\n"\
"In other words to get a higher score one should remove as many\n"\
"blocks as possible in each move.\n"\
"\n"\
"\n"\
"Have fun!\n";
  PangoFontDescription *font_desc;

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW (window), "Help");
  gtk_grab_add(window);
  gtk_window_set_position(GTK_WINDOW(window),GTK_WIN_POS_CENTER);
  gtk_widget_set_usize(window, 520, 440);
  gtk_window_set_resizable (GTK_WINDOW(window),TRUE);
  gtk_container_border_width(GTK_CONTAINER(window), 5);
  gtk_signal_connect_object(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(modal_close), GTK_OBJECT(window));
  gtk_signal_connect_object(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(modal_close), GTK_OBJECT(window));

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_container_border_width(GTK_CONTAINER(vbox), 5);
  gtk_container_add(GTK_CONTAINER(window), vbox);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX (vbox), hbox, TRUE, TRUE, 0);

  scrolled = gtk_scrolled_window_new(NULL, NULL);

  help = gtk_text_view_new ();
  gtk_text_view_set_editable(GTK_TEXT_VIEW(help), FALSE);
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled), help);

  gtk_box_pack_start(GTK_BOX(hbox), scrolled, TRUE, TRUE, 0);

  separator = gtk_hseparator_new();
  gtk_box_pack_start(GTK_BOX(vbox), separator, FALSE, FALSE, 5);  
  
  button_box = gtk_hbox_new(TRUE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), button_box, FALSE, TRUE, 0);

  button_close = gtk_button_new_with_label("Close");
  gtk_box_pack_start(GTK_BOX(button_box), button_close, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS(button_close, GTK_CAN_DEFAULT);
  gtk_widget_grab_default(button_close);
  gtk_signal_connect_object(GTK_OBJECT(button_close),"clicked",GTK_SIGNAL_FUNC(modal_close),GTK_OBJECT(window));

  font_desc = pango_font_description_from_string ("Fixed 11");
  gtk_widget_modify_font (help, font_desc);
  pango_font_description_free (font_desc);

  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (help));
  gtk_text_buffer_set_text(buffer, help_text, -1);

  gtk_widget_show_all(window);
}

void game_over_dialog(void)
{
  GtkWidget *window;
  GtkWidget *boxv1;
  GtkWidget *button_close;
  GtkWidget *text;
  GtkWidget *separator;
  gchar buf[40];
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_grab_add(window);
  gtk_window_set_position(GTK_WINDOW(window),GTK_WIN_POS_CENTER);
  gtk_window_set_title(GTK_WINDOW(window),"Game Over");
  gtk_container_border_width(GTK_CONTAINER(window), 10);

  gtk_signal_connect_object(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(modal_close), GTK_OBJECT(window));
  gtk_signal_connect_object(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(modal_close), GTK_OBJECT(window));

  boxv1 = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(window), boxv1);

  snprintf (buf, 39, "Game over, your score : %i", score);
  text = gtk_label_new(buf);
  gtk_box_pack_start(GTK_BOX(boxv1), text, FALSE, FALSE, 0);  

  if (atoi(scores[9]) < score) {
    read_scores ();
    write_scores ();
    text = gtk_label_new("You are in top 10!");
    gtk_box_pack_start(GTK_BOX(boxv1), text, FALSE, FALSE, 0);  
  }

  separator = gtk_hseparator_new();
  gtk_box_pack_start(GTK_BOX(boxv1), separator, FALSE, FALSE, 10);  

  button_close = gtk_button_new_with_label("Close");
  gtk_box_pack_start(GTK_BOX(boxv1), button_close, FALSE, FALSE, 0);  
  GTK_WIDGET_SET_FLAGS(button_close, GTK_CAN_DEFAULT);
  gtk_widget_grab_default(button_close);
  gtk_signal_connect_object(GTK_OBJECT(button_close), "clicked", GTK_SIGNAL_FUNC(modal_close), GTK_OBJECT(window));

  gtk_widget_show_all(window); 
}

static GtkItemFactoryEntry menu_items[] = {
  {"/_Game",                NULL,         NULL,          0, "<Branch>"},
  {"/Game/_New Game",	    "<control>N", new_game,      0, NULL},
  {"/Game/_Restart Game",   "<control>R", restart_game,  0, NULL},
  {"/Game/sep1",            NULL,         NULL,          0, "<Separator>"},
  {"/Game/_Undo",           "<control>Z", undo_game,     0, NULL},
  {"/Game/_High Scores...", NULL,         scores_game,   0, NULL},
  {"/Game/_Information...", NULL,         info_game,     0, NULL},
  {"/Game/sep2",            NULL,         NULL,          0, "<Separator>"},
  {"/Game/E_xit",           "<control>X", exit_game,     0, NULL},
  {"/_Options",             NULL,         NULL,          0, "<Branch>"},
  {"/Options/_Board Size",  NULL,         NULL,          0, "<Branch>"},
  {"/Options/Board Size/Normal", NULL,    size_normal,   0, "<RadioItem>"},
  {"/Options/Board Size/Large", NULL,     size_large,    0, "/Options/Board Size/Normal"},
  {"/Options/_Tiles Style", NULL,         NULL,          0, "<Branch>"},
  {"/Options/Tiles Style/Letters", NULL,  style_letters, 0, "<RadioItem>"},
  {"/Options/Tiles Style/Balls", NULL,    style_balls,   0, "/Options/Tiles Style/Letters"},
  {"/_Help",                NULL,         NULL,          0, "<LastBranch>"},
  {"/_Help/_About...",      NULL,         about_game,    0, NULL},
  {"/_Help/_Help...",       "F1",         help_game,     0, NULL}
};


void get_main_menu (GtkWidget  *window, GtkWidget **menubar)
{
  GtkItemFactory *item_factory;
  GtkAccelGroup *accel_group;
  gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);

  accel_group = gtk_accel_group_new ();
  item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", accel_group);
  gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, NULL);
  gtk_window_add_accel_group (GTK_WINDOW(window), accel_group);

  if (menubar)
    *menubar = gtk_item_factory_get_widget (item_factory, "<main>");
}

gint drawing_board_expose_event(GtkWidget *widget, GdkEventConfigure *event)
{  
  draw_board();
  return FALSE; /* propagate the signal */
}

gint drawing_board_button_press_event(GtkWidget *widget, GdkEventConfigure *event)
{
  int x, y, i, j, k = 0;
  gdk_window_get_pointer(widget->window, &x, &y, NULL);
  x = x / 32;
  y = y / 32;
  if (board[x][y] > 5) {
    make_undo ();
    for (i = 0; i < width; i++)
      for (j = 0; j < height; j++)
        if (board[i][j] > 5) {
          board[i][j] = 0;
          k++;
        }
    score += calc_score (k);
    mark = 0;
    for (i = 0; i < width; i++)
      for (j = height - 1; j > 0; j--)
        for (k = j ; k > 0; k--)
	  if (board[i][j] == 0) board_shift_down (i, j);
    for (i = 0; i < width - 1; i++)
      for (k = width - i; k > 0; k--)
        if (board[i][height - 1] == 0) board_shift_left (i);

    draw_board ();
    check_game_over ();
    update_status ();
    if (game_over) game_over_dialog ();
  }
  else
    if (board[x][y] <= 5 && board[x][y] != 0) {
      for (i = 0; i < width; i++)
        for (j = 0; j < height; j++)
          if (board[i][j] > 5) board[i][j] -=5;
      if ((mark = board_select (x, y, board[x][y])) == 1) {
        board[x][y] -= 5;
        mark = 0;
      }
      draw_board ();
      update_status ();
    }
  return FALSE; /* propagate the signal */	
}


int main(int argc, char *argv[])
{
  GtkWidget *window;
  GtkWidget *main_vbox;
  GtkWidget *menubar;
  GdkPixmap *pixmap;
  GdkBitmap *bitmap;

  gtk_init (&argc, &argv);
  
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_signal_connect (GTK_OBJECT (window), "destroy", 
                      GTK_SIGNAL_FUNC (exit_game), 
                      "WM destroy");
  gtk_window_set_title (GTK_WINDOW(window), "Same Game");
  gtk_window_set_policy (GTK_WINDOW(window), 0, 0, 1);  

  main_vbox = gtk_vbox_new (FALSE, 1);
  gtk_container_border_width (GTK_CONTAINER (main_vbox), 1);
  gtk_container_add (GTK_CONTAINER (window), main_vbox);
  gtk_widget_show (main_vbox);
  
  get_main_menu (window, &menubar);
  gtk_box_pack_start (GTK_BOX (main_vbox), menubar, FALSE, TRUE, 0);
  gtk_widget_show (menubar);

  drawing_board = gtk_drawing_area_new ();
  gtk_drawing_area_size (GTK_DRAWING_AREA (drawing_board), width*32, height*32);
  gtk_box_pack_start (GTK_BOX (main_vbox), drawing_board, FALSE, TRUE, 0);
  gtk_widget_set_events(drawing_board, GDK_BUTTON_PRESS_MASK | GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK);
  gtk_widget_show (drawing_board);

  gtk_signal_connect(GTK_OBJECT(drawing_board), "expose_event", GTK_SIGNAL_FUNC (drawing_board_expose_event), NULL);
  gtk_signal_connect(GTK_OBJECT(drawing_board), "button_press_event", GTK_SIGNAL_FUNC(drawing_board_button_press_event), NULL);                                

  status_bar = gtk_statusbar_new ();
  context_id = gtk_statusbar_get_context_id (GTK_STATUSBAR (status_bar), "Score");
  gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, "Welcome to Same Game");
  gtk_box_pack_start (GTK_BOX (main_vbox), status_bar, FALSE, TRUE, 0);
  gtk_widget_show (status_bar);

  gtk_widget_realize (window);
  pixmap = gdk_pixmap_create_from_xpm_d (window->window, &bitmap, NULL, gsame_xpm);
  gdk_window_set_icon (window->window, NULL, pixmap, bitmap);

  gtk_widget_show (window);

  make_new ();

  tiles[0] = gdk_pixmap_create_from_xpm_d (window->window, NULL, NULL, null_xpm);
  tiles[1] = gdk_pixmap_create_from_xpm_d (window->window, NULL, NULL, a1_xpm);
  tiles[2] = gdk_pixmap_create_from_xpm_d (window->window, NULL, NULL, b1_xpm);
  tiles[3] = gdk_pixmap_create_from_xpm_d (window->window, NULL, NULL, c1_xpm);
  tiles[4] = gdk_pixmap_create_from_xpm_d (window->window, NULL, NULL, d1_xpm);
  tiles[5] = gdk_pixmap_create_from_xpm_d (window->window, NULL, NULL, e1_xpm);
  tiles[6] = gdk_pixmap_create_from_xpm_d (window->window, NULL, NULL, a1_mark_xpm);
  tiles[7] = gdk_pixmap_create_from_xpm_d (window->window, NULL, NULL, b1_mark_xpm);
  tiles[8] = gdk_pixmap_create_from_xpm_d (window->window, NULL, NULL, c1_mark_xpm);
  tiles[9] = gdk_pixmap_create_from_xpm_d (window->window, NULL, NULL, d1_mark_xpm);
  tiles[10] = gdk_pixmap_create_from_xpm_d (window->window, NULL, NULL, e1_mark_xpm);
  tiles[11] = gdk_pixmap_create_from_xpm_d (window->window, NULL, NULL, a2_xpm);
  tiles[12] = gdk_pixmap_create_from_xpm_d (window->window, NULL, NULL, b2_xpm);
  tiles[13] = gdk_pixmap_create_from_xpm_d (window->window, NULL, NULL, c2_xpm);
  tiles[14] = gdk_pixmap_create_from_xpm_d (window->window, NULL, NULL, d2_xpm);
  tiles[15] = gdk_pixmap_create_from_xpm_d (window->window, NULL, NULL, e2_xpm);
  tiles[16] = gdk_pixmap_create_from_xpm_d (window->window, NULL, NULL, a2_mark_xpm);
  tiles[17] = gdk_pixmap_create_from_xpm_d (window->window, NULL, NULL, b2_mark_xpm);
  tiles[18] = gdk_pixmap_create_from_xpm_d (window->window, NULL, NULL, c2_mark_xpm);
  tiles[19] = gdk_pixmap_create_from_xpm_d (window->window, NULL, NULL, d2_mark_xpm);
  tiles[20] = gdk_pixmap_create_from_xpm_d (window->window, NULL, NULL, e2_mark_xpm);

  gtk_main ();
  
  return(0);
}
